package cn.estsh.i3plus.core.apiservice.mq;

import cn.estsh.i3plus.core.api.iservice.busi.ISysMessageService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysUserService;
import cn.estsh.i3plus.core.apiservice.util.MailUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.JsonUtilTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysMessage;
import cn.estsh.i3plus.pojo.platform.bean.SysRefUserMessage;
import cn.estsh.i3plus.pojo.platform.bean.SysUser;
import cn.estsh.impp.framework.boot.util.RedisCacheTool;
import com.alibaba.fastjson.JSON;
import com.rabbitmq.client.Channel;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.context.annotation.Configuration;

import java.io.IOException;

/**
 * @Description : 邮件队列处理
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2018-11-15 22:15
 * @Modify:
 **/
@ConditionalOnExpression("'${impp.mq.queue.mail:true}' == 'true'")
@Configuration
public class MessageMailQueueReceiver {
    private static final Logger LOGGER = LoggerFactory.getLogger(MessageMailQueueReceiver.class);

    @Autowired
    private ISysMessageService sysMessageService;
    @Autowired
    private ISysUserService sysUserService;
    @Autowired
    private MailUtil mailUtil;

    /**
     * 系统邮件处理队列
     *
     * @param data    系统消息信息
     * @param channel 通道
     * @param message 消息
     */
    @RabbitListener(queues = PlatformConstWords.QUEUE_IMPP_MESSAGE_MAIL)
    public void processImppMail(String data, Channel channel, Message message) {
        SysMessage msg = JsonUtilTool.decode(data, SysMessage.class);
        Long startTime = System.currentTimeMillis();
        try {
            LOGGER.debug("【MQ-QUEUE_IMPP_MESSAGE_MAIL】数据接收成功：{}", msg);
            if (msg.getMessageSendTime() == null) {
                msg.setMessageSendTime(TimeTool.getNowTime(true));
            }

            // 持久化消息化消息并初始化状态
            ConvertBean.serviceModelInitialize(msg, msg.getMessageSenderNameRdd());
            msg.setSendStatus(ImppEnumUtil.MESSAGE_SEND_STATUS.SENDING.getValue());
            msg = sysMessageService.insertSysMessage(msg);

            // 默认发送成功
            msg.setSendStatus(ImppEnumUtil.MESSAGE_SEND_STATUS.SEND_SUCCESS.getValue());

            // 设置邮件主题内容及收件人
            mailUtil.init();
            mailUtil.setSubject(msg.getMessageTitle());
            mailUtil.setContentType(ImppEnumUtil.MESSAGE_TYPE_CONTENT.valueOfDescription(msg.getMessageContentType()));
            mailUtil.setBody(msg.getMessageContent());

            // 判断收件人类型
            if (msg.getMessageReceiverType() != null) {
                if (msg.getMessageReceiverType().intValue() == ImppEnumUtil.MESSAGE_RECEIVER_TYPE.URGENT.getValue()) {
                    // 判断是否为系统紧急提示 微服注册状态提示
                    mailUtil.setTo(RedisCacheTool.getSysConfigStrVal(PlatformConstWords.CONTACT_MAIL));
                    try {
                        mailUtil.send();
                    } catch (Exception e) {
                        saveSendErrorMsg(msg, RedisCacheTool.getSysConfigStrVal(PlatformConstWords.CONTACT_MAIL), e);
                    }

                } else if (msg.getMessageReceiverType().intValue() == ImppEnumUtil.MESSAGE_RECEIVER_TYPE.EXTERNAL.getValue()) {
                    //判断是否为外部邮件
                    mailUtil.setTo(StringUtils.split(msg.getMessageReceiversNameRdd(), ","));
                    try {
                        mailUtil.send();
                    } catch (Exception e) {
                        saveSendErrorMsg(msg, RedisCacheTool.getSysConfigStrVal(PlatformConstWords.CONTACT_MAIL), e);
                    }

                } else if (msg.getMessageReceiverType().intValue() == ImppEnumUtil.MESSAGE_RECEIVER_TYPE.INTERNAL.getValue()) {
                    // 收件人信息
                    String[] messageReceiver = msg.getMessageReceiversId().split(",");

                    SysRefUserMessage refUserMessage;
                    SysUser sysUser;

                    for (int i = 0; i < messageReceiver.length; i++) {
                        sysUser = sysUserService.getSysUserById(Long.parseLong(messageReceiver[i]));

                        refUserMessage = new SysRefUserMessage();
                        refUserMessage.setMessageId(msg.getId());
                        refUserMessage.setMessageSoftType(msg.getMessageSoftType());
                        refUserMessage.setMessageTitleRdd(msg.getMessageTitle());
                        refUserMessage.setMessageTypeRdd(msg.getMessageType());
                        refUserMessage.setMessageSenderNameRdd(msg.getMessageSenderNameRdd());
                        refUserMessage.setReceiverId(sysUser.getId());
                        refUserMessage.setReceiverNameRdd(sysUser.getUserName());
                        refUserMessage.setMessageStatus(ImppEnumUtil.MESSAGE_STATUS.UNREAD.getValue());
                        refUserMessage.setReceiverTime(TimeTool.getNowTime(true));
                        refUserMessage.setIsUrgent(msg.getIsUrgent());

                        sysMessageService.insertSysRefUserMessage(refUserMessage);

                        mailUtil.setTo(sysUser.getUserEmail());

                        try {
                            mailUtil.send();
                        } catch (Exception e) {
                            saveSendErrorMsg(msg, RedisCacheTool.getSysConfigStrVal(PlatformConstWords.CONTACT_MAIL), e);
                        }
                    }
                }
            }

            //信息已处理
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
        } catch (Exception e) {
            LOGGER.error("【MQ-QUEUE_IMPP_MESSAGE_MAIL】处理出错：{}", e.getMessage(), e);
            //丢弃这条消息
            try {
//                if(msg.getId() != null) {
//                    sysMessageService.deleteSysMessageById(msg.getId());
//                }
                // 未成功处理，重新发送
                //channel.basicNack(message.getMessageProperties().getDeliveryTag(),false,true);
                //todo 此处对邮件持久化，通过参数设置，是否还需要重复重新发送
                // 改为接收成功
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        } finally {
            LOGGER.info("[Core Email] Email Id:{} , 耗时{}, Message Info :{}", msg.getId(), System.currentTimeMillis() - startTime, JSON.toJSONString(msg));
        }
    }

    /**
     * 保存发送异常信息
     *
     * @param message 发送消息主体
     * @param mark    异常消息标识
     * @param e       异常信息
     */
    private void saveSendErrorMsg(SysMessage message, String mark, Exception e) {
        if (message == null) {
            return;
        }
        // 更新消息发送状态
        message.setSendStatus(ImppEnumUtil.MESSAGE_SEND_STATUS.SEND_FAIL.getValue());

        StringBuilder errorMsg = new StringBuilder();
        // 校验标识是否存在
        if (StringUtils.isNotBlank(mark)) {
            errorMsg.append("发送标识【");
            errorMsg.append(mark);
            errorMsg.append("】");
        }

        // 拼接异常信息
        errorMsg.append(ExceptionUtils.getMessage(e));
        errorMsg.append("\r\n");

        if (!StringUtils.isBlank(message.getErrorMessage())) {
            errorMsg.insert(0, message.getErrorMessage());
        }

        message.setErrorMessage(errorMsg.toString());
    }
}
